# -*- coding: utf-8 -*-
from setuphelpers import *
import winshell
import os

uninstallkey = []

# Declaring specific app values (TO CHANGE)
bin_name_string = "putty-%s-installer.msi"


def remove_putty_if_install_with_exe():
    if uninstall_key_exists("PuTTY_is1"):
        killalltasks(control.impacted_process.split(","))
        # uninstaller cannot be silenced by design (cf putty.iss), so we remove stuff by hand
        # run('"%s" /VERYSILENT /SUPPRESSMSGBOXES' % os.path.join(programfiles32,'putty','unins000.exe'))
        if isdir(install_location("PuTTY_is1")):
            remove_tree(install_location("PuTTY_is1"))
        registry_delete(HKEY_LOCAL_MACHINE, "SOFTWARE\\Classes\\PuTTYPrivateKey", None)
        if iswin64():
            registry_delete(HKEY_LOCAL_MACHINE, "SOFTWARE\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\PuTTY_is1", None)
        else:
            registry_delete(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\PuTTY_is1", None)
        putty_link = makepath(winshell.programs(common=1), "PuTTY")
        if isdir(putty_link):
            remove_tree(putty_link)
        unregister_uninstall("Putty_is1")


def install():
    # Initializing variables
    package_version = control.version.split("-", 1)[0]
    bin_name = bin_name_string % package_version

    # Installing the package
    remove_putty_if_install_with_exe()

    print("Installing: %s" % bin_name)
    install_msi_if_needed(bin_name, min_version=package_version)

    #Uninstall old version putty
    for msi in installed_softwares('putty'):
        if not 'msiexec' in str(uninstall_cmd(msi['key'])).lower() :
            continue
        if Version(msi['version']) < package_version:
            run(uninstall_cmd(msi['key']))



def uninstall():
    # Uninstalling the package
    remove_putty_if_install_with_exe()


def update_package():
    print("Downloading/Updating package content from upstream binary sources")

    # Initializing variables
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    app_name = control.name
    app_arch = control.architecture
    url = control.sources
    bin_start = bin_name_string.split("%s")[0]
    # Translation arch
    if app_arch == "x64":
        url_dl_part = "https://the.earth.li/~sgtatham/putty/latest/w64/"
    else:
        url_dl_part = "https://the.earth.li/~sgtatham/putty/latest/w32/"

    # Getting latest version from official website
    print("URL used is: " + url)
    for read_bs in bs_find_all(url, "code", proxies=proxies):
        if read_bs.text.startswith(bin_start):
            latest_bin = read_bs.text
            version = latest_bin.split("-")[-2]
            if ("arm" in latest_bin) or ("64bit" in latest_bin.lower()):
                continue
            break
    url_dl = url_dl_part + latest_bin

    print("Latest %s version is: %s" % (app_name, version))
    print("Download url is: %s" % url_dl)

    # Downloading latest binaries
    if not isfile(latest_bin):
        print("Downloading: %s" % latest_bin)
        wget(url_dl, latest_bin, proxies=proxies)

        # Checking version from file
        version_from_file = get_msi_properties(latest_bin)["ProductName"].split(" ")[2]
        if version != version_from_file:
            os.rename(latest_bin, bin_name_string % version_from_file)
            version = version_from_file

        # Changing version of the package
        control.version = "%s-%s" % (version, int(control.version.split("-")[-1]) + 1)
        control.save_control_to_wapt()
        print("Changing version to: %s in WAPT\\control" % control.version)

    # Deleting outdated binaries
    remove_outdated_binaries(version)


def get_proxies():
    import platform

    if platform.python_version_tuple()[0] == "3":
        from urllib.request import getproxies
    else:
        from urllib import getproxies
    return getproxies()


def get_version_from_binary(filename):
    if filename.endswith(".msi"):
        return get_msi_properties(filename)["ProductVersion"]
    else:
        return get_file_properties(filename)["ProductVersion"]


def remove_outdated_binaries(
    version, list_filename_contain=("x64", "x86", "win32", "win64", "32", "64"), list_extensions=("exe", "msi", "deb", "rpm", "dmg", "pkg")
):
    for file_ext in list_extensions:
        for bin_in_dir in glob.glob("*.%s" % file_ext):
            if not version in bin_in_dir:
                for filename_contain in list_filename_contain:
                    if not filename_contain in bin_in_dir:
                        print("Outdated binary: %s Removed" % bin_in_dir)
                        remove_file(bin_in_dir)


def bs_find_all(url, element, attribute=None, value=None, headers=None, proxies=None, features="html.parser", **kwargs):
    """ "You may need to use a header for some websites. For example: headers={'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:78.0) Gecko/20100101 Firefox/78.0'}"""
    import requests

    page = requests.get(url, proxies=proxies, headers=headers, **kwargs).text
    try:
        import bs4 as BeautifulSoup

        soup = BeautifulSoup.BeautifulSoup(page, features=features)
    except:
        import BeautifulSoup

        soup = BeautifulSoup.BeautifulSoup(page)
    if value:
        return soup.findAll(element, {attribute: value})
    else:
        return soup.findAll(element)
